VERSION 5.00
Begin VB.Form frmMain 
   Caption         =   "Form1"
   ClientHeight    =   3195
   ClientLeft      =   60
   ClientTop       =   345
   ClientWidth     =   4680
   LinkTopic       =   "Form1"
   ScaleHeight     =   3195
   ScaleWidth      =   4680
   StartUpPosition =   3  'Windows Default
End
Attribute VB_Name = "frmMain"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'---------------------------------
'Title: DirectX Tutorial
'
'Description:
'
'Author: Jacob Roman
'
'Date: 12/04/2005
'
'Contact: jacob_c_roman@hotmail.com
'---------------------------------

Option Explicit

Private Type RECT_SINGLE

    Left As Single
    Top As Single
    Right As Single
    Bottom As Single

End Type

Private Type Animation_Type
    
    Number_Of_Frames As Long
    Current_Frame As Single
    Frame_Counter As Long
    Speed As Single
    Frame_Size As RECT
    
End Type

Private Type Sprite_Type

    X As Single
    Y As Single
    Animation_State As Animation_Type
    
End Type


'The 2D (Transformed and Lit) vertex format type.
Private Type TLVERTEX

    X As Single
    Y As Single
    Z As Single
    RHW As Single
    Color As Long
    Specular As Long
    TU As Single
    TV As Single
    
End Type

Private Const ANIMATION_MODE_SINGLE_SHOT As Long = 0
Private Const ANIMATION_MODE_LOOP As Long = 1

'Some color depth constants to help make the DX constants more readable.
Private Const COLOR_DEPTH_16_BIT As Long = D3DFMT_R5G6B5
Private Const COLOR_DEPTH_24_BIT As Long = D3DFMT_A8R8G8B8
Private Const COLOR_DEPTH_32_BIT As Long = D3DFMT_X8R8G8B8

'The 2D (Transformed and Lit) vertex format.
Private Const FVF_TLVERTEX As Long = D3DFVF_XYZRHW Or D3DFVF_TEX1 Or D3DFVF_DIFFUSE Or D3DFVF_SPECULAR

Private DirectX8 As DirectX8 'The master DirectX object.
Private Direct3D As Direct3D8 'Controls all things 3D.
Private Direct3D_Device As Direct3DDevice8 'Represents the hardware rendering.
Private Direct3DX As D3DX8

Private Window_Width As Long
Private Window_Height As Long

Private Fullscreen_Width As Long
Private Fullscreen_Height As Long
Private Color_Depth As Long

Private Fullscreen_Enabled As Boolean 'Helps determine whether it's fullscreen mode.
Private Running As Boolean 'Helps determine whether the main game loop is running.

Private Vertex_List(3) As TLVERTEX '4 vertices will make a square.

Private Texture() As Direct3DTexture8

Private Sprite As Sprite_Type

Private Scalar As D3DVECTOR2

Private Frame As RECT_SINGLE

Private Sub Window_Setup(Window As Form, Optional ByVal X As Long = -1, Optional ByVal Y As Long = -1, Optional ByVal Width As Long = -1, Optional ByVal Height As Long = -1, Optional Caption As String = " ", Optional Auto_Redraw As Boolean = False, Optional ByVal Back_Color As Long = -1)
    
    'Use -1 for default values and "" for default strings.
    
    With Window
    
        If Caption <> " " Then .Caption = Caption 'Else use current setting. Note: Some
                                                  'people may want "" as the caption.
        .AutoRedraw = Auto_Redraw
        .ScaleMode = 3
        If X <> -1 Then .Left = X 'Else use current setting.
        If Y <> -1 Then .Top = Y 'Else use current setting.
        If Width <> -1 Then .Width = Width * Screen.TwipsPerPixelX 'Else use current setting.
        If Height <> -1 Then .Height = Height * Screen.TwipsPerPixelY 'Else use current setting.
        If Back_Color <> -1 Then .BackColor = Back_Color 'Else use current setting.
        .Show
        .Refresh
        .SetFocus
        
    End With
    
End Sub

'This function will make it much easier to setup the vertices with the info it needs.
Private Function Create_TLVertex(X As Single, Y As Single, Z As Single, RHW As Single, Color As Long, Specular As Long, TU As Single, TV As Single) As TLVERTEX

    Create_TLVertex.X = X
    Create_TLVertex.Y = Y
    Create_TLVertex.Z = Z
    Create_TLVertex.RHW = RHW
    Create_TLVertex.Color = Color
    Create_TLVertex.Specular = Specular
    Create_TLVertex.TU = TU
    Create_TLVertex.TV = TV
    
End Function

Private Function DirectX_Initialize() As Boolean

    On Error GoTo Error_Handler
    
    Dim Display_Mode As D3DDISPLAYMODE 'Display mode desciption.
    Dim Direct3D_Window As D3DPRESENT_PARAMETERS 'Backbuffer and viewport description.
    
    Set DirectX8 = New DirectX8 'Creates the DirectX object.
    Set Direct3D = DirectX8.Direct3DCreate() 'Creates the Direct3D object using the DirectX object.
    Set Direct3DX = New D3DX8
    
    If Fullscreen_Enabled = True Then
    
        'Now that we are working with fullscreen mode, we must set up the
        'screen resolution to switch to, rather than use the default screen
        'resolution.
        
        Display_Mode.Width = Fullscreen_Width
        Display_Mode.Height = Fullscreen_Height
        Display_Mode.Format = Color_Depth
    
        Direct3D_Window.Windowed = False 'The app will be in fullscreen mode.
        Direct3D_Window.BackBufferCount = 1 '1 backbuffer only
        Direct3D_Window.BackBufferWidth = Display_Mode.Width 'Match the backbuffer width with the display width
        Direct3D_Window.BackBufferHeight = Display_Mode.Height 'Match the backbuffer height with the display height
        Direct3D_Window.hDeviceWindow = frmMain.hWnd 'Use frmMain as the device window.
        
    Else
    
        Direct3D.GetAdapterDisplayMode D3DADAPTER_DEFAULT, Display_Mode 'Use the current display mode that you
                                                                        'are already on. Incase you are confused, I'm
                                                                        'talking about your current screen resolution. ;)
        
        Direct3D_Window.Windowed = True 'The app will be in windowed mode.
    
    End If
    
    Direct3D_Window.SwapEffect = D3DSWAPEFFECT_COPY_VSYNC 'Refresh when the monitor does.
    Direct3D_Window.BackBufferFormat = Display_Mode.Format 'Sets the format that was retrieved into the backbuffer.
    
    'Creates the rendering device with some useful info, along with the info
    'we've already setup for Direct3D_Window.
    Set Direct3D_Device = Direct3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, frmMain.hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, Direct3D_Window)
    
    Direct3D_Device.SetVertexShader FVF_TLVERTEX 'Set the type of vertex shading. (Required)
    
    'Sets up properties for alphablending.
    Direct3D_Device.SetRenderState D3DRS_SRCBLEND, D3DBLEND_ONE
    Direct3D_Device.SetRenderState D3DRS_DESTBLEND, D3DBLEND_SRCALPHA
    
    'Sets up properties for transparency.
    Direct3D_Device.SetRenderState D3DRS_ALPHAREF, 255
    Direct3D_Device.SetRenderState D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL
    
    'These lines are not needed, but it's nice to be able to filter the
    'textures to make them look nicer.
    
    Direct3D_Device.SetTextureStageState 0, D3DTSS_MINFILTER, D3DTEXF_POINT
    Direct3D_Device.SetTextureStageState 0, D3DTSS_MAGFILTER, D3DTEXF_POINT

    Exit Function
    
Error_Handler:
    
    MsgBox "An error occured while initializing DirectX", vbCritical

    DirectX_Initialize = False


End Function

Private Sub Load_Textures()

    Dim File_Path(10) As String
    Dim Width As Long
    Dim Height As Long
    Dim Transparency_Color As Long
    Dim Current_Texture As Long
    
    ReDim Texture(10) As Direct3DTexture8
    
    File_Path(0) = App.Path & "\Graphics\Sprite1.bmp"
    File_Path(1) = App.Path & "\Graphics\Sprite2.bmp"
    File_Path(2) = App.Path & "\Graphics\Sprite3.bmp"
    File_Path(3) = App.Path & "\Graphics\Sprite4.bmp"
    File_Path(4) = App.Path & "\Graphics\Sprite5.bmp"
    File_Path(5) = App.Path & "\Graphics\Sprite6.bmp"
    File_Path(6) = App.Path & "\Graphics\Sprite7.bmp"
    File_Path(7) = App.Path & "\Graphics\Sprite8.bmp"
    File_Path(8) = App.Path & "\Graphics\Sprite9.bmp"
    File_Path(9) = App.Path & "\Graphics\Sprite10.bmp"
    File_Path(10) = App.Path & "\Graphics\Sprite11.bmp"

    Width = 256
    Height = 256
    
    Transparency_Color = D3DColorRGBA(0, 0, 0, 255)
    
    For Current_Texture = 0 To 10

        Set Texture(Current_Texture) = Direct3DX.CreateTextureFromFileEx(Direct3D_Device, _
                                                        File_Path(Current_Texture), _
                                                        Width, Height, _
                                                        0, _
                                                        0, _
                                                        D3DFMT_A8R8G8B8, _
                                                        D3DPOOL_MANAGED, _
                                                        D3DX_FILTER_POINT, _
                                                        D3DX_FILTER_POINT, _
                                                        Transparency_Color, _
                                                        ByVal 0, _
                                                        ByVal 0)
                                                        
    Next Current_Texture

End Sub

Private Sub Setup_Sprite()
    
    Sprite.Animation_State.Number_Of_Frames = 11
    
    Sprite.X = 130
    Sprite.Y = 90
    
    If Fullscreen_Enabled = False Then
    
        Scalar.X = 1
        Scalar.Y = 1
        
    Else
    
        Scalar.X = Fullscreen_Width / Window_Width
        Scalar.Y = Fullscreen_Height / Window_Height
        
    End If
    
    Sprite.Animation_State.Speed = 0.2

    Sprite.Animation_State.Frame_Size.Left = 0
    Sprite.Animation_State.Frame_Size.Top = 0
    Sprite.Animation_State.Frame_Size.Right = 50
    Sprite.Animation_State.Frame_Size.Bottom = 50

End Sub

Private Sub Animate_Sprite()

    Dim Current_Frame As Single
    
    Dim Transparency_Color As Long
    
    'Right here will alphablend the polygon
    Direct3D_Device.SetRenderState D3DRS_ALPHABLENDENABLE, False

    'Right here will give the polygon transparency
    Direct3D_Device.SetRenderState D3DRS_ALPHATESTENABLE, True
    
            
    If Int(Sprite.Animation_State.Current_Frame) > (Sprite.Animation_State.Number_Of_Frames - 1) Then
    
        Sprite.Animation_State.Current_Frame = 0
    
    End If
    
    Current_Frame = Int(Sprite.Animation_State.Current_Frame)
    
    Transparency_Color = D3DColorRGBA(255, 255, 255, 255)
    
    Frame.Left = (Sprite.X + Sprite.Animation_State.Frame_Size.Left) * Scalar.X
    Frame.Top = (Sprite.Y + Sprite.Animation_State.Frame_Size.Top) * Scalar.Y
    Frame.Right = (Sprite.X + Sprite.Animation_State.Frame_Size.Right) * Scalar.X
    Frame.Bottom = (Sprite.Y + Sprite.Animation_State.Frame_Size.Bottom) * Scalar.Y
    
    'Creates the polygon.
    '---------------------------------------------------------------
    With Frame
    
        Vertex_List(0) = Create_TLVertex(.Left, .Top, 0, 1, Transparency_Color, 0, 0, 0)
        Vertex_List(1) = Create_TLVertex(.Right, .Top, 0, 1, Transparency_Color, 0, 1, 0)
        Vertex_List(2) = Create_TLVertex(.Left, .Bottom, 0, 1, Transparency_Color, 0, 0, 1)
        Vertex_List(3) = Create_TLVertex(.Right, .Bottom, 0, 1, Transparency_Color, 0, 1, 1)
    
    End With
    '---------------------------------------------------------------
                                     
    'Set the texture
    Direct3D_Device.SetTexture 0, Texture(Current_Frame)
    
    Sprite.Animation_State.Current_Frame = Sprite.Animation_State.Current_Frame + Sprite.Animation_State.Speed
    
End Sub

Private Sub Game_Loop()

    Do While Running = True
        
        DoEvents 'Allow events to happen so the program doesn't lock up.
        
        '----------------------------------------------------
        'DirectX automatically handles the framerate for you
        'which makes it run (at most) as fast as the monitors
        'refresh rate, so you don't need to add extra code to
        'slow down the loop and run at a certain number of frames
        'per second.
        '----------------------------------------------------
        
        'Clears the backbuffer.
        Direct3D_Device.Clear 0, ByVal 0, D3DCLEAR_TARGET, D3DColorRGBA(0, 0, 0, 0), 1#, 0
            
            Direct3D_Device.BeginScene
                
                Animate_Sprite
            
                'Draw the polygon
                Direct3D_Device.DrawPrimitiveUP D3DPT_TRIANGLESTRIP, 2, Vertex_List(0), Len(Vertex_List(0))
            
            Direct3D_Device.EndScene
        
        'Flips the backbuffer into the form window.
        Direct3D_Device.Present ByVal 0, ByVal 0, 0, ByVal 0
        
    Loop

End Sub

Private Sub Close_Program()

    Running = False 'This helps the program bail out of the game loop.
    
    'Unload all of the DirectX objects.
    
    Set Direct3D_Device = Nothing
    Set Direct3D = Nothing
    Set DirectX8 = Nothing
    
    Unload Me 'Unload the form.
    
    End 'Ends the program.
    
    'Although the Unload statement located above exits the program, you
    'will end up with an Automation error after doing so. The End statement
    'will help prevent that, and end the app completely.

End Sub

Private Sub Form_Activate()

    Window_Width = 320
    Window_Height = 240
    
    Fullscreen_Width = 640
    Fullscreen_Height = 480
    Color_Depth = COLOR_DEPTH_32_BIT

    Window_Setup frmMain, -1, -1, Window_Width, Window_Height, "DirectX Tutorial", , RGB(0, 0, 0)
    
    DirectX_Initialize 'Initializes DirectX and Direct3D.
    
    Load_Textures 'Loads textures from files.
    
    Setup_Sprite
    
    Running = True 'Initializations all set. It's now ok to activate the game loop.
    
    Game_Loop

End Sub

Private Sub Form_KeyDown(KeyCode As Integer, Shift As Integer)

    If KeyCode = vbKeyEscape Then 'If the user presses the Esc key...
    
        Close_Program
    
    End If

End Sub

Private Sub Form_Load()

    'This event will fire before the form has completely loaded.
    
    If MsgBox("Click Yes to go to full screen (Recommended)", vbQuestion Or vbYesNo, "Options") = vbYes Then Fullscreen_Enabled = True

End Sub

Private Sub Form_Unload(Cancel As Integer)

    Close_Program

End Sub
