VERSION 5.00
Begin VB.Form frmMain 
   Caption         =   "Form1"
   ClientHeight    =   3195
   ClientLeft      =   60
   ClientTop       =   345
   ClientWidth     =   4680
   LinkTopic       =   "Form1"
   ScaleHeight     =   3195
   ScaleWidth      =   4680
   StartUpPosition =   3  'Windows Default
End
Attribute VB_Name = "frmMain"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'---------------------------------
'Title: DirectX Tutorial
'
'Description:
'
'Author: Jacob Roman
'
'Date: 12/02/2005
'
'Contact: jacob_c_roman@hotmail.com
'---------------------------------

Option Explicit

'The 2D (Transformed and Lit) vertex format type.
Private Type TLVERTEX

    X As Single
    Y As Single
    Z As Single
    RHW As Single
    Color As Long
    Specular As Long
    TU As Single
    TV As Single
    
End Type

Private Type Map_Type

    Map() As String
    Tile() As Long
    New_Tile() As Long
    Width As Long
    Height As Long
    New_Width As Long
    New_Height As Long
    Number_Of_Vertices_Per_Tile_Set() As Long
    Number_Of_Polygons_Per_Tile_Set() As Long
    Number_Of_Textures As Long
    Texture_List() As Direct3DTexture8
    Number_Of_Vertices As Long

End Type

Private Declare Function QueryPerformanceCounter Lib "Kernel32" (lpPerformanceCount As Currency) As Long
Private Declare Function QueryPerformanceFrequency Lib "Kernel32" (lpPerformanceCount As Currency) As Long

'Some color depth constants to help make the DX constants more readable.
Private Const COLOR_DEPTH_16_BIT As Long = D3DFMT_R5G6B5
Private Const COLOR_DEPTH_24_BIT As Long = D3DFMT_A8R8G8B8
Private Const COLOR_DEPTH_32_BIT As Long = D3DFMT_X8R8G8B8

'The 2D (Transformed and Lit) vertex format.
Private Const FVF_TLVERTEX As Long = D3DFVF_XYZRHW Or D3DFVF_TEX1 Or D3DFVF_DIFFUSE Or D3DFVF_SPECULAR

Private Const TILE_WIDTH As Single = 31.2
Private Const TILE_HEIGHT As Single = 26.4

Private Const Number_Of_Vertices_Per_Quad As Long = 6
Private Const Number_Of_Triangles_Per_Quad As Long = 2

Private DirectX8 As DirectX8 'The master DirectX object.
Private Direct3D As Direct3D8 'Controls all things 3D.
Private Direct3D_Device As Direct3DDevice8 'Represents the hardware rendering.
Private Direct3DX As D3DX8

Private Fullscreen_Width As Long
Private Fullscreen_Height As Long

Private Fullscreen_Enabled As Boolean 'Helps determine whether it's fullscreen mode.
Private Running As Boolean 'Helps determine whether the main game loop is running.

Private Master_Vertex_List() As TLVERTEX
Private Vertex_List() As TLVERTEX
Private Sorted_Vertex_List() As TLVERTEX

Private Vertex_Buffer As Direct3DVertexBuffer8

Private Current_Texture As Long

Private GlobalX As Single, GlobalY As Single
Private Initial_Global As D3DVECTOR2

Private Map As Map_Type

Private Scalar As D3DVECTOR2

Private Time(3) As Single
Private Milliseconds(3) As Single
Private Ticks_Per_Second As Currency
Private Start_Time As Currency

Private Direct_Input As DirectInput8

Private Keyboard_Device As DirectInputDevice8
Private Keyboard_State As DIKEYBOARDSTATE

Private Function Hi_Res_Timer_Initialize() As Boolean

    If QueryPerformanceFrequency(Ticks_Per_Second) = 0 Then

        Hi_Res_Timer_Initialize = False

    Else
    
        QueryPerformanceCounter Start_Time
     
        Hi_Res_Timer_Initialize = True
     
    End If

End Function

Private Function Get_Elapsed_Time() As Single
    
    Dim Last_Time As Currency
    
    Dim Current_Time As Currency

    QueryPerformanceCounter Current_Time
    
    Get_Elapsed_Time = (Current_Time - Last_Time) / Ticks_Per_Second
    
    QueryPerformanceCounter Last_Time
    
End Function


'This function will make it much easier to setup the vertices with the info it needs.
Private Function Create_TLVertex(X As Single, Y As Single, Z As Single, RHW As Single, Color As Long, Specular As Long, TU As Single, TV As Single) As TLVERTEX

    Create_TLVertex.X = X
    Create_TLVertex.Y = Y
    Create_TLVertex.Z = Z
    Create_TLVertex.RHW = RHW
    Create_TLVertex.Color = Color
    Create_TLVertex.Specular = Specular
    Create_TLVertex.TU = TU
    Create_TLVertex.TV = TV
    
End Function

Private Function DirectX_Initialize() As Boolean

    On Error GoTo Error_Handler
    
    Dim Display_Mode As D3DDISPLAYMODE 'Display mode desciption.
    Dim Direct3D_Window As D3DPRESENT_PARAMETERS 'Backbuffer and viewport description.
    
    Set DirectX8 = New DirectX8 'Creates the DirectX object.
    Set Direct3D = DirectX8.Direct3DCreate() 'Creates the Direct3D object using the DirectX object.
    Set Direct3DX = New D3DX8
    
    If Fullscreen_Enabled = True Then
    
        'Now that we are working with fullscreen mode, we must set up the
        'screen resolution to switch to, rather than use the default screen
        'resolution.
        
        Display_Mode.Width = Fullscreen_Width
        Display_Mode.Height = Fullscreen_Height
        Display_Mode.Format = COLOR_DEPTH_16_BIT
    
        Direct3D_Window.Windowed = False 'The app will be in fullscreen mode.
        Direct3D_Window.BackBufferCount = 1 '1 backbuffer only
        Direct3D_Window.BackBufferWidth = Display_Mode.Width 'Match the backbuffer width with the display width
        Direct3D_Window.BackBufferHeight = Display_Mode.Height 'Match the backbuffer height with the display height
        Direct3D_Window.hDeviceWindow = frmMain.hWnd 'Use frmMain as the device window.
        
        Scalar.X = Fullscreen_Width / frmMain.ScaleWidth
        Scalar.Y = Fullscreen_Height / frmMain.ScaleHeight
        
    Else
    
        Direct3D.GetAdapterDisplayMode D3DADAPTER_DEFAULT, Display_Mode 'Use the current display mode that you
                                                                        'are already on. Incase you are confused, I'm
                                                                        'talking about your current screen resolution. ;)
        
        Direct3D_Window.Windowed = True 'The app will be in windowed mode.
        
        Scalar.X = 1
        Scalar.Y = 1
    
    End If
    
    Direct3D_Window.SwapEffect = D3DSWAPEFFECT_COPY_VSYNC 'Refresh when the monitor does.
    Direct3D_Window.BackBufferFormat = Display_Mode.Format 'Sets the format that was retrieved into the backbuffer.
    
    'Creates the rendering device with some useful info, along with the info
    'we've already setup for Direct3D_Window.
    Set Direct3D_Device = Direct3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, frmMain.hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, Direct3D_Window)
    
    Direct3D_Device.SetVertexShader FVF_TLVERTEX 'Set the type of vertex shading. (Required)
    
    'Sets up properties for transparency.
    Direct3D_Device.SetRenderState D3DRS_ALPHAREF, 255
    Direct3D_Device.SetRenderState D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL
    
    'These lines are not needed, but it's nice to be able to filter the
    'textures to make them look nicer.
    
    Direct3D_Device.SetTextureStageState 0, D3DTSS_MINFILTER, D3DTEXF_POINT
    Direct3D_Device.SetTextureStageState 0, D3DTSS_MAGFILTER, D3DTEXF_POINT

    Exit Function
    
Error_Handler:
    
    MsgBox "An error occured while initializing DirectX", vbCritical
    
    Close_Program
    
    DirectX_Initialize = False


End Function

Private Sub Setup_Map()
    
    Dim X As Long, Y As Long
    
    Dim Temp As Variant
    
    Map.Width = (20) - 1
    Map.Height = (16) - 1
    
    ReDim Map.Map(Map.Height) As String
    
    Map.Map(0) = "2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2"
    Map.Map(1) = "0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0"
    Map.Map(2) = "0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0"
    Map.Map(3) = "0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0"
    Map.Map(4) = "0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0"
    Map.Map(5) = "0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0"
    Map.Map(6) = "0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0"
    Map.Map(7) = "0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0"
    Map.Map(8) = "0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0"
    Map.Map(9) = "0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0"
    Map.Map(10) = "0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0"
    Map.Map(11) = "0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0"
    Map.Map(12) = "0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0"
    Map.Map(13) = "0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0"
    Map.Map(14) = "0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0"
    Map.Map(15) = "2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2"
    
    ReDim Map.Tile(Map.Width, Map.Height) As Long
    ReDim Map.New_Tile(Map.Width, Map.Height) As Long
    
    For Y = 0 To Map.Height
    
        Temp = Split(Map.Map(Y), ", ", -1)
    
        For X = 0 To Map.Width
    
            Map.Tile(X, Y) = CLng(Temp(X))
            
        Next X
        
    Next Y

End Sub

Private Sub Load_Textures()

    Dim File_Path() As String
    Dim Width As Long
    Dim Height As Long
    Dim Transparency_Color As Long
    
    Map.Number_Of_Textures = (7) - 1
    
    ReDim File_Path(Map.Number_Of_Textures) As String
    ReDim Map.Texture_List(Map.Number_Of_Textures) As Direct3DTexture8
    ReDim Map.Number_Of_Vertices_Per_Tile_Set(Map.Number_Of_Textures) As Long
    ReDim Map.Number_Of_Polygons_Per_Tile_Set(Map.Number_Of_Textures) As Long
    
    File_Path(0) = App.Path & "\Graphics\Grass.bmp"
    File_Path(1) = App.Path & "\Graphics\Plant.bmp"
    File_Path(2) = App.Path & "\Graphics\Sign.bmp"
    File_Path(3) = App.Path & "\Graphics\Pond1.bmp"
    File_Path(4) = App.Path & "\Graphics\Pond2.bmp"
    File_Path(5) = App.Path & "\Graphics\Pond3.bmp"
    File_Path(6) = App.Path & "\Graphics\Pond4.bmp"

    Width = 256
    Height = 256
    
    Transparency_Color = D3DColorRGBA(0, 0, 0, 255)

    For Current_Texture = 0 To Map.Number_Of_Textures

        Set Map.Texture_List(Current_Texture) = Direct3DX.CreateTextureFromFileEx(Direct3D_Device, _
                                                        File_Path(Current_Texture), _
                                                        Width, Height, _
                                                        0, _
                                                        0, _
                                                        D3DFMT_A8R8G8B8, _
                                                        D3DPOOL_MANAGED, _
                                                        D3DX_FILTER_POINT, _
                                                        D3DX_FILTER_POINT, _
                                                        Transparency_Color, _
                                                        ByVal 0, _
                                                        ByVal 0)
                                                        
    Next Current_Texture

End Sub

Private Sub Map_To_Vertex_List_And_Clip(Map As Map_Type, Old_Vertex_List() As TLVERTEX, New_Vertex_List() As TLVERTEX)

    Dim X As Long, Y As Long
    Dim X2 As Long, Y2 As Long
    Dim I As Long, J As Long
    Dim Temp As Long
    
    Temp = -1
    
    For Y = 0 To Map.Height
 
        For X = 0 To Map.Width
            
            Old_Vertex_List(I + 0) = Create_TLVertex(GlobalX + ((TILE_WIDTH * X) + 0) * Scalar.X, GlobalY + ((TILE_HEIGHT * Y) + 0) * Scalar.Y, 0, 1, D3DColorRGBA(255, 255, 255, 0), 0, 0, 0)
            Old_Vertex_List(I + 1) = Create_TLVertex(GlobalX + ((TILE_WIDTH * X) + TILE_WIDTH) * Scalar.X, GlobalY + ((TILE_HEIGHT * Y) + 0) * Scalar.Y, 0, 1, D3DColorRGBA(255, 255, 255, 0), 0, 1, 0)
            Old_Vertex_List(I + 2) = Create_TLVertex(GlobalX + ((TILE_WIDTH * X) + 0) * Scalar.X, GlobalY + ((TILE_HEIGHT * Y) + TILE_HEIGHT) * Scalar.Y, 0, 1, D3DColorRGBA(255, 255, 255, 0), 0, 0, 1)
            Old_Vertex_List(I + 3) = Create_TLVertex(GlobalX + ((TILE_WIDTH * X) + TILE_WIDTH) * Scalar.X, GlobalY + ((TILE_HEIGHT * Y) + 0) * Scalar.Y, 0, 1, D3DColorRGBA(255, 255, 255, 0), 0, 1, 0)
            Old_Vertex_List(I + 4) = Create_TLVertex(GlobalX + ((TILE_WIDTH * X) + TILE_WIDTH) * Scalar.X, GlobalY + ((TILE_HEIGHT * Y) + TILE_HEIGHT) * Scalar.Y, 0, 1, D3DColorRGBA(255, 255, 255, 0), 0, 1, 1)
            Old_Vertex_List(I + 5) = Create_TLVertex(GlobalX + ((TILE_WIDTH * X) + 0) * Scalar.X, GlobalY + ((TILE_HEIGHT * Y) + TILE_HEIGHT) * Scalar.Y, 0, 1, D3DColorRGBA(255, 255, 255, 0), 0, 0, 1)

            If Clip_Polygon(Old_Vertex_List(I + 0), Old_Vertex_List(I + 4)) = False Then

                New_Vertex_List(J + 0) = Old_Vertex_List(I + 0)
                New_Vertex_List(J + 1) = Old_Vertex_List(I + 1)
                New_Vertex_List(J + 2) = Old_Vertex_List(I + 2)
                New_Vertex_List(J + 3) = Old_Vertex_List(I + 3)
                New_Vertex_List(J + 4) = Old_Vertex_List(I + 4)
                New_Vertex_List(J + 5) = Old_Vertex_List(I + 5)

                X2 = X2 + 1
    
                If Temp <> Y Then
    
                    Temp = Y
                    X2 = 1
                    Y2 = Y2 + 1
    
                End If
                
                Map.New_Tile(X2 - 1, Y2 - 1) = Map.Tile(X, Y)
                
                J = J + Number_Of_Vertices_Per_Quad
    
            End If

            I = I + Number_Of_Vertices_Per_Quad
            
        Next X
        
    Next Y

    Map.New_Width = X2
    Map.New_Height = Y2

End Sub

Private Sub Sort_Polygons_By_Texture(Map As Map_Type, Old_Vertex_List() As TLVERTEX, New_Vertex_List() As TLVERTEX)

    Dim X As Long, Y As Long
    
    Dim I As Long, J As Long, K As Long

    For Current_Texture = 0 To Map.Number_Of_Textures
        
        I = 0
        
        For Y = 0 To Map.New_Height - 1
        
            For X = 0 To Map.New_Width - 1
                
                    If Map.New_Tile(X, Y) = Current_Texture Then
                        
                         New_Vertex_List(J + 0) = Old_Vertex_List(I + 0)
                         New_Vertex_List(J + 1) = Old_Vertex_List(I + 1)
                         New_Vertex_List(J + 2) = Old_Vertex_List(I + 2)
                         New_Vertex_List(J + 3) = Old_Vertex_List(I + 3)
                         New_Vertex_List(J + 4) = Old_Vertex_List(I + 4)
                         New_Vertex_List(J + 5) = Old_Vertex_List(I + 5)
                         
                         'Sum all vertices, don't reset this variable..
                         J = J + Number_Of_Vertices_Per_Quad
                         'Sum vertices per texture..
                         K = K + Number_Of_Vertices_Per_Quad
                         
                    End If
                    
                    I = I + Number_Of_Vertices_Per_Quad
                    
            Next X
            
        Next Y
        
        Map.Number_Of_Polygons_Per_Tile_Set(Current_Texture) = K / 3
        Map.Number_Of_Vertices_Per_Tile_Set(Current_Texture) = K
        
        K = 0 'Reset this one, anothr texture is coming
            
    Next Current_Texture

End Sub

Private Function Clip_Polygon(Polygon_A As TLVERTEX, Polygon_B As TLVERTEX) As Boolean
    
    Dim R As RECT
    Dim Border_Size As Long
    
    Border_Size = 0
    
    If Fullscreen_Enabled = False Then
    
        R.Left = Border_Size * Scalar.X
        R.Top = Border_Size * Scalar.Y
        R.Right = Me.ScaleWidth - Border_Size * Scalar.X
        R.bottom = Me.ScaleHeight - Border_Size * Scalar.Y
        
    Else

        R.Left = Border_Size * Scalar.X
        R.Top = Border_Size * Scalar.Y
        R.Right = Fullscreen_Width - Border_Size * Scalar.X
        R.bottom = Fullscreen_Height - Border_Size * Scalar.Y

    End If
    
    If (Polygon_A.X < R.Left And Polygon_B.X < R.Left) Or (Polygon_A.X > R.Right And Polygon_B.X > R.Right) Or _
           (Polygon_A.Y < R.Top And Polygon_B.Y < R.Top) Or (Polygon_A.Y > R.bottom And Polygon_B.Y > R.bottom) Then Clip_Polygon = True

End Function

Private Sub Create_Polygons()
    
    Map.Number_Of_Vertices = ((Map.Width + 1) * (Map.Height + 1) * Number_Of_Vertices_Per_Quad)
    
    ReDim Master_Vertex_List(Map.Number_Of_Vertices - 1) As TLVERTEX
    ReDim Vertex_List(Map.Number_Of_Vertices - 1) As TLVERTEX
    ReDim Sorted_Vertex_List(Map.Number_Of_Vertices - 1) As TLVERTEX
    
    Map_To_Vertex_List_And_Clip Map, Master_Vertex_List(), Vertex_List()
    
    Sort_Polygons_By_Texture Map, Vertex_List(), Sorted_Vertex_List()

    'Creates the vertex buffer.
    Set Vertex_Buffer = Direct3D_Device.CreateVertexBuffer(Len(Sorted_Vertex_List(0)) * Map.Number_Of_Vertices, 0, FVF_TLVERTEX, D3DPOOL_MANAGED)
    D3DVertexBuffer8SetData Vertex_Buffer, 0, Len(Sorted_Vertex_List(0)) * Map.Number_Of_Vertices, 0, Sorted_Vertex_List(0)
    
    Direct3D_Device.SetStreamSource 0, Vertex_Buffer, Len(Sorted_Vertex_List(0))
    
End Sub

Private Sub DirectInput_Initialize_Keyboard(Window As Form)
    
    Set Direct_Input = DirectX8.DirectInputCreate
    Set Keyboard_Device = Direct_Input.CreateDevice("GUID_SysKeyboard")
    Keyboard_Device.SetCommonDataFormat DIFORMAT_KEYBOARD
    Keyboard_Device.SetCooperativeLevel Window.hWnd, DISCL_BACKGROUND Or DISCL_NONEXCLUSIVE
    Keyboard_Device.Acquire
    Keyboard_Device.GetDeviceStateKeyboard Keyboard_State

End Sub

Private Function DirectInput_Key_State(Key_Code As Long) As Long

    Keyboard_Device.GetDeviceStateKeyboard Keyboard_State
    
    DirectInput_Key_State = Keyboard_State.Key(Key_Code)

End Function

Private Sub Control()

    Const Speed As Single = 75
    
    Dim Elapsed_Time As Single
    
    Elapsed_Time = Get_Elapsed_Time
    
    If DirectInput_Key_State(DIK_ESCAPE) <> 0 Then Close_Program
    
    '---------------------------------------------
    'Left/Right
    '---------------------------------------------
    If DirectInput_Key_State(DIK_LEFT) <> 0 And DirectInput_Key_State(DIK_RIGHT) = 0 Then
    
        GlobalX = Initial_Global.X + Speed * Time(0) * Scalar.X
        
    ElseIf DirectInput_Key_State(DIK_LEFT) = 0 And DirectInput_Key_State(DIK_RIGHT) <> 0 Then
    
        GlobalX = Initial_Global.X - Speed * Time(1) * Scalar.X
        
    ElseIf (DirectInput_Key_State(DIK_LEFT) <> 0 And DirectInput_Key_State(DIK_RIGHT) <> 0) Or _
           (DirectInput_Key_State(DIK_LEFT) = 0 And DirectInput_Key_State(DIK_RIGHT) = 0) Then
           
        Milliseconds(0) = Elapsed_Time
        Milliseconds(1) = Elapsed_Time
        Initial_Global.X = GlobalX
        
    End If
    '---------------------------------------------
    
    '---------------------------------------------
    'Up/Down
    '---------------------------------------------
    If DirectInput_Key_State(DIK_UP) <> 0 And DirectInput_Key_State(DIK_DOWN) = 0 Then
    
        GlobalY = Initial_Global.Y + Speed * Time(2) * Scalar.Y
        
    ElseIf DirectInput_Key_State(DIK_UP) = 0 And DirectInput_Key_State(DIK_DOWN) <> 0 Then
    
        GlobalY = Initial_Global.Y - Speed * Time(3) * Scalar.Y
        
    ElseIf (DirectInput_Key_State(DIK_UP) <> 0 And DirectInput_Key_State(DIK_DOWN) <> 0) Or _
           (DirectInput_Key_State(DIK_UP) = 0 And DirectInput_Key_State(DIK_DOWN) = 0) Then
           
        Milliseconds(2) = Elapsed_Time
        Milliseconds(3) = Elapsed_Time
        Initial_Global.Y = GlobalY

    End If
    '---------------------------------------------

End Sub

Private Sub Collision_Detection()

    Dim R As RECT
    
    R.Left = (-(Map.Width * TILE_WIDTH / 2) - TILE_WIDTH / 2) * Scalar.X
    R.Top = (-(Map.Height * TILE_HEIGHT / 2) - TILE_HEIGHT / 2) * Scalar.Y
    R.Right = 0
    R.bottom = 0
    
    If GlobalX <= R.Left Then GlobalX = R.Left
    If GlobalY <= R.Top Then GlobalY = R.Top
    If GlobalX >= R.Right Then GlobalX = R.Right
    If GlobalY >= R.bottom Then GlobalY = R.bottom

End Sub

Private Sub Game_Loop()
    
    Dim Number_Of_Polygons As Long
    
    Dim Total_Number_Of_Vertices As Long
    
    Dim I As Long
    
    Number_Of_Polygons = (Map.Width + 1) * (Map.Height + 1) * Number_Of_Triangles_Per_Quad
    
    Do While Running = True
        
        DoEvents 'Allow events to happen so the program doesn't lock up.
        
        '----------------------------------------------------
        'DirectX automatically handles the framerate for you
        'which makes it run (at most) as fast as the monitors
        'refresh rate, so you don't need to add extra code to
        'slow down the loop and run at a certain number of frames
        'per second.
        '----------------------------------------------------
        
        For I = 0 To 3
        
            Time(I) = Get_Elapsed_Time - Milliseconds(I)
        
        Next I
        
        Control
        
        Collision_Detection
        
        Create_Polygons
        
        'Clears the backbuffer.
        Direct3D_Device.Clear 0, ByVal 0, D3DCLEAR_TARGET, D3DColorRGBA(0, 0, 0, 0), 1#, 0
            
            Direct3D_Device.BeginScene
                
                'Direct3D_Device.SetRenderState D3DRS_FILLMODE, 2
                Direct3D_Device.SetRenderState D3DRS_ALPHATESTENABLE, False
                
                Total_Number_Of_Vertices = 0
                
                For Current_Texture = 0 To Map.Number_Of_Textures
                
                    Direct3D_Device.SetTexture 0, Map.Texture_List(Current_Texture)
                    Direct3D_Device.DrawPrimitive D3DPT_TRIANGLELIST, Total_Number_Of_Vertices, Map.Number_Of_Polygons_Per_Tile_Set(Current_Texture)
                    
                    Total_Number_Of_Vertices = Total_Number_Of_Vertices + Map.Number_Of_Vertices_Per_Tile_Set(Current_Texture)
                    
                Next Current_Texture
                
            Direct3D_Device.EndScene
        
        'Flips the backbuffer into the form window.
        Direct3D_Device.Present ByVal 0, ByVal 0, 0, ByVal 0
        
        QueryPerformanceCounter Start_Time
        
    Loop

End Sub

Private Sub Close_Program()

    Running = False 'This helps the program bail out of the game loop.
    
    'Unload all of the DirectX objects.
    
    For Current_Texture = 0 To Map.Number_Of_Textures
    
        Set Map.Texture_List(Current_Texture) = Nothing
    
    Next Current_Texture
    
    Set Direct3DX = Nothing
    Set Direct3D_Device = Nothing
    Set Direct3D = Nothing
    Set DirectX8 = Nothing
    
    Unload Me 'Unload the form.
    
    End 'Ends the program.
    
    'Although the Unload statement located above exits the program, you
    'will end up with an Automation error after doing so. The End statement
    'will help prevent that, and end the app completely.

End Sub

Private Sub Form_Activate()
    
    Dim I As Long
    
    Fullscreen_Width = 800
    Fullscreen_Height = 600

    With frmMain
    
        .Caption = "DirectX Tutorial"
        .ScaleMode = 3
        .Width = (640 / 2) * Screen.TwipsPerPixelX
        .Height = (480 / 2) * Screen.TwipsPerPixelY
    
    End With
        
    DirectX_Initialize 'Initializes DirectX and Direct3D.
    
    Setup_Map
    
    Load_Textures 'Loads the textures from file.
    
    Create_Polygons 'Creates the polygon.
    
    DirectInput_Initialize_Keyboard frmMain
    
    Hi_Res_Timer_Initialize
    
    For I = 0 To 3
    
        Milliseconds(I) = Get_Elapsed_Time
    
    Next I
    Running = True 'Initializations all set. It's now ok to activate the game loop.
    
    Game_Loop

End Sub

Private Sub Form_Load()

    'This event will fire before the form has completely loaded.
    
    If MsgBox("Click Yes to go to full screen (Recommended)", vbQuestion Or vbYesNo, "Options") = vbYes Then Fullscreen_Enabled = True

End Sub

Private Sub Form_Unload(Cancel As Integer)

    Close_Program

End Sub

Private Function Split(ByVal sIn As String, Optional sDelim As String, Optional nLimit As Long = -1, Optional bCompare As VbCompareMethod = vbBinaryCompare) As Variant
    
    'For those who have VB5
    
    Dim sRead As String, sOut() As String, nC As Integer
    
    If sDelim = "" Then
    
        Split = sIn
        
    End If
    
    sRead = ReadUntil(sIn, sDelim, bCompare)
    
    Do
    
        ReDim Preserve sOut(nC)
        
        sOut(nC) = sRead
        nC = nC + 1
        
        If nLimit <> -1 And nC >= nLimit Then Exit Do
        
        sRead = ReadUntil(sIn, sDelim)
        
    Loop While sRead <> ""
    
    ReDim Preserve sOut(nC)
    
    sOut(nC) = sIn
    Split = sOut
    
End Function

Private Function ReadUntil(ByRef sIn As String, sDelim As String, Optional bCompare As VbCompareMethod = vbBinaryCompare) As String
    
    'For those who have VB5
    
    Dim nPos As String
    
    nPos = InStr(1, sIn, sDelim, bCompare)
    
    If nPos > 0 Then
    
        ReadUntil = Left(sIn, nPos - 1)
        sIn = Mid(sIn, nPos + Len(sDelim))
        
    End If
    
End Function
